﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CfChart
{
    /// <summary>
    /// 平面直角坐标系图表
    /// </summary>
    /// <remarks>该类是使用平面直角坐标系的图表的基类，像柱形图、折线图等</remarks>
    public abstract class AxisChart : Chart
    {
        #region protected Fields

        /// <summary>
        /// 原点
        /// </summary>
        protected Point origin;

        /// <summary>
        /// 坐标系宽度
        /// </summary>
        protected int axisWidth;

        /// <summary>
        /// 坐标系高度
        /// </summary>
        protected int axisHeight;

        /// <summary>
        /// 总系列数
        /// </summary>
        protected int seriesCount;

        /// <summary>
        /// 总序列数
        /// </summary>
        protected int serialCount;

        /// <summary>
        /// 最大刻度值
        /// </summary>
        protected double[] maxScale;

        /// <summary>
        /// 最小刻度值
        /// </summary>
        protected double[] minScale;

        /// <summary>
        /// 最大值
        /// </summary>
        protected double[] maxValue;

        /// <summary>
        /// 最小值
        /// </summary>
        protected double[] minValue;

        #endregion

        /// <summary>
        /// 绘制图表
        /// </summary>
        protected override void DrawChart()
        {
            this.PrepareData();

            this.DrawAxis();
        }

        /// <summary>
        /// 绘制图例
        /// </summary>
        protected override void DrawLegend()
        {
            for (int i = 0; i < this.seriesCount; i += 2)
            {
                Label label = new Label
                                  {
                                      Location = new Point(30, this.origin.Y + 43 + i * 20),
                                      Size = new Size(90, 20),
                                      Text = DataSeries[i].Label
                                  };
                Controls.Add(label);
                graphics.FillRectangle(new SolidBrush(DataSeries[i].Color), 3, label.Top + 2, 25, label.Height - 5);

                label = new Label
                            {
                                Location = new Point(Width / 2 + 30, this.origin.Y + 43 + i * 20),
                                Size = new Size(90, 20),
                                Text = DataSeries[i + 1].Label
                            };
                Controls.Add(label);
                graphics.FillRectangle(new SolidBrush(DataSeries[i + 1].Color), 123, label.Top + 2, 25, label.Height - 5);
            }
        }

        /// <summary>
        /// 设置颜色
        /// </summary>
        protected override void SetColorForData()
        {
            this.seriesCount = DataSeries.Count;
            Color[] colors = new PredefinedColors(this.seriesCount).Colors;
            for (int i = 0; i < this.seriesCount; i++)
            {
                DataSeries[i].Color = colors[i];
            }
        }

        /// <summary>
        /// 准备数据
        /// </summary>
        protected void PrepareData()
        {
            this.seriesCount = DataSeries.Count;
            this.serialCount = DataSeries[0].Serials.Count;
            this.origin = new Point(40, 110);

            #region 遍历得到各系列数据的最大值和最小值
            this.maxValue = new double[this.seriesCount];
            this.minValue = new double[this.seriesCount];
            for (int i1 = 0; i1 < this.seriesCount; i1++)
            {
                this.maxValue[i1] = this.minValue[i1] = DataSeries[i1].Serials[0].Data;
                for (int j1 = 0; j1 < this.serialCount; j1++)
                {
                    this.maxValue[i1] = DataSeries[i1].Serials[j1].Data > this.maxValue[i1] ? DataSeries[i1].Serials[j1].Data : this.maxValue[i1];
                    this.minValue[i1] = DataSeries[i1].Serials[j1].Data < this.minValue[i1] ? DataSeries[i1].Serials[j1].Data : this.minValue[i1];
                }
            }
            #endregion

            // 得到最大刻度值和最小刻度值
            this.maxScale = new double[this.seriesCount];
            this.minScale = new double[this.seriesCount];
            GetScaleScope(this.maxValue, this.minValue, ref this.maxScale, ref this.minScale);

            // 修正原点坐标
            this.origin.X = Math.Max(this.maxScale[0].ToString().Length, this.minScale[0].ToString().Length) * 11;
            this.axisWidth = Width - this.origin.X - 10;
            this.axisHeight = 100;
        }

        /// <summary>
        /// 绘制坐标轴及水平刻度线、水平刻度值
        /// </summary>
        protected void DrawAxis()
        {
            Pen axisPen = new Pen(Color.DarkGray, 1);

            // 绘制y轴
            graphics.DrawLine(axisPen, origin.X, origin.Y + 2, origin.X, origin.Y - axisHeight);

            #region 绘制x轴及水平刻度线、水平刻度值
            double scale = (maxScale[0] - minScale[0]) / 4;
            for (int i2 = 0; i2 < 5; i2++)
            {
                int y = origin.Y - i2 * 25;
                graphics.DrawLine(axisPen, origin.X - 3, y, origin.X - 3 + axisWidth, y);

                // y轴坐标值
                Label label = new Label
                {
                    Text = (minScale[0] + i2 * scale).ToString(),
                    TextAlign = ContentAlignment.TopRight,
                    Location = new Point(3, y - 5),
                    Size = new Size(origin.X - 6, 20)
                };
                Controls.Add(label);
            }
            #endregion
        }

        /// <summary>
        /// 获取刻度范围
        /// </summary>
        /// <param name="maxValue">最大值</param>
        /// <param name="minValue">最小值</param>
        /// <param name="maxScale">最大刻度值</param>
        /// <param name="minScale">最小刻度值</param>
        protected static void GetScaleScope(double[] maxValue, double[] minValue, ref double[] maxScale, ref double[] minScale)
        {
            for (int i = 0; i < maxValue.Length; i++)
            {
                string firstNonZeroFigureInMax = maxValue[i].ToString().Replace(".", "").Replace("0", "")[0].ToString();
                string firstNonZeroFigureInMin = minValue[i].ToString().Replace(".", "").Replace("0", "")[0].ToString();
                double rate = Math.Pow(10, Math.Floor(Math.Log10(maxValue[i])));

                maxScale[i] = (double.Parse(firstNonZeroFigureInMax) + 0.5) * rate;
                minScale[i] = (double.Parse(firstNonZeroFigureInMin) - 0.5) * rate;

                if (maxScale[i] < maxValue[i])
                {
                    maxScale[i] = (double.Parse(firstNonZeroFigureInMax) + 1) * rate;
                }

                if (minScale[i] > minValue[i])
                {
                    minScale[i] = (double.Parse(firstNonZeroFigureInMin) - 1) * rate;
                }
            }
        }

        /// <summary>
        /// 获取序列的高度
        /// </summary>
        /// <param name="axisHeight">坐标轴高度（总高度）</param>
        /// <param name="maxScale">最大刻度值</param>
        /// <param name="minScale">最小刻度值</param>
        /// <param name="value">序列值</param>
        /// <returns>序列的高度</returns>
        protected static int GetSerialHeight(int axisHeight, double maxScale, double minScale, double value)
        {
            return Convert.ToInt32(Math.Ceiling((value - minScale) * axisHeight / (maxScale - minScale) + 0.5));
        }
    }
}
